// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

// __Please ask for permission, if you want to use some code from this shader__

// __Discord: The Alexander # 3744__

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

varying vec4 color;
varying highp vec3 wpos;
varying vec3 cotf;
varying highp vec3 p;
varying highp vec3 position;

#ifdef FOG
varying vec4 fogColor;
#endif

#include "uniformShaderConstants.h"
#include "util.h"

uniform vec4 FOG_COLOR;
uniform vec2 FOG_CONTROL;
uniform float RENDER_DISTANCE;
uniform highp float TIME;

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

#include "set/sl_noise.h"
#include "set/player_shadow.h"

vec3 ToneMap(vec3 x) {
    float A = 2.40;
    float B = 0.05;
    float C = 2.40;
    float D = 0.40;
    float E = 0.40;
    float F = 1.2;
    
    x=x*(A*x+B)/(x*(C*x+D)+E/F);
  
return (x-dot(x,vec3(0.1)))*1.18+0.09;}

vec3 saturate(vec3 rgb, float adjustment){
const vec3 W = vec3(0.2125, 0.7154, 0.0721);
vec3 intensity = vec3(dot(rgb, W));
return mix(intensity, rgb, adjustment);
}

vec3 czm_exposure(vec3 diffuse)
{
float gamma = 0.0;
float exposure = 1.7;
diffuse *= exposure/(1. + diffuse / exposure);
//.diffuse = pow(diffuse, vec3(1. / gamma));
return diffuse;
}

float map(float value, float min1, float max1, float min2, float max2) {
  return min2 + (value - min1) * (max2 - min2) / (max1 - min1);
}

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0); 
   vec4 limits = texture2D_AA(TEXTURE_1, vec2(0,1));
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
   vec4 limits = texture2D(TEXTURE_1, vec2(0,1));
#endif
	
#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif
	
vec4 inColor = color;

//Water

highp float wave = sin(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.z+p.z*1.0)))+cos(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.
z+p.z*1.0)))+tan(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.z+p.z*1.0)))+sin(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.z+p.z*1.0)))+cos(noise(vec2(TIME*1.2+p.y+p.z+p.x+p.x+p.x*2.0,TIME*3.2+p.x+p.z+p.y+p.x+p.y*1.1+p.y+p.z+p.z+p.z*1.0)));

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse *= texture2D( TEXTURE_1, uv1 );
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif
	
	diffuse.rgb *= inColor.rgb;
#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

//Direccion Light

highp vec3 dirlek = normalize(cross(dFdx(p),dFdy(p)));

float dir = max(dirlek.x, -dirlek.x);

diffuse.rgb *= mix(vec3(1.25), vec3(0.88),dir);

float area = smoothstep( .885, .875, uv1.y);
vec3 kena = mix(vec3(0.5), vec3(1.0), uv1.x);
diffuse.rgb *= mix(vec3(1.0),kena,  area);

//Dectectors

bool unwater = FOG_CONTROL.x==0.0;
float rain = smoothstep(0.66, 0.3, FOG_CONTROL.x);
bool water = (color.b>color.r);
bool nether = (FOG_COLOR.r>FOG_COLOR.b && FOG_COLOR.r<0.5 && FOG_COLOR.b<0.2);
bool soul_nether = (FOG_COLOR.r<FOG_COLOR.b && FOG_COLOR.r<0.1 && FOG_COLOR.b<0.2);

//Fog

vec3 fog = mix(diffuse.rgb, FOG_COLOR.rgb,clamp(length(-wpos)/RENDER_DISTANCE*0.4,0.0,0.4));

diffuse.rgb = fog;

//Rain effect

float rain_detect = 1.0-pow(FOG_CONTROL.y, 11.0);

diffuse.rgb *= vec3(1.5,1.5,1.5)+vec3(-0.9,-0.9,-0.9)*rain_detect+uv1.x*vec3(1.5,1.5,1.5)*rain_detect;

//TimeCycle

vec4 detc = texture2D(TEXTURE_1, vec2(0.0,1.0));
float day = pow(min(max(1.1-pow(detc.r,10.0),0.0),1.0),3.0);
float night = min(pow(1.0-detc.r,2.0)*2.0,1.0);

diffuse.rgb *= vec3(1.0);
vec3 colorN = vec3(0.6,0.6,1.1);
colorN = mix(colorN, vec3(0.8), uv1.x);
diffuse.rgb *= mix(vec3(1.0),colorN,night);

#if defined(BLEND) || defined(FOG)
if(water==true){
diffuse.a = 0.600;//Opacitiy

diffuse.rgb *= vec3(0.2);//Cleared texture base

//Color in Day
vec3 clor = mix(vec3(0.1,0.3,0.8), vec3(1.8), clamp(length(wpos)/RENDER_DISTANCE*0.5,0.0,0.5));

//Color in Night
vec3 clorN = mix(vec3(0.19,0.3,0.6), vec3(1.5), clamp(length(wpos)/RENDER_DISTANCE*0.3,0.0,0.3));

//Color in rain
vec3 clorR = mix(vec3(0.5,0.5,0.5), vec3(1.0), clamp(length(wpos)/RENDER_DISTANCE*2.0,0.0,2.0));

//activators
vec3 lor = mix(clor,clorN, night);
diffuse.rgb += mix(lor,clorR, rain);

highp float time = TIME*1.33;
highp float wwave = noise(time+position.xz+position.zx+position.xy);
diffuse.rgb += wwave*0.098833;

diffuse.rgb *= vec3(0.0);
diffuse.rgb += vec3(0.1,0.5,1.0);
diffuse.rgb += noise(wpos*0.5*abs(wave))*0.8;
diffuse.rgb *= mix(vec3(0.8),vec3(0.7),(wave));

if(nether==false){
if(FOG_COLOR.r*1.0>FOG_COLOR.b){

diffuse.rgb += clamp(1.0-length(-wpos.z*13.0)/RENDER_DISTANCE*2.0,0.0,2.0)*vec3(2.0,1.78,1.0)*length(-wpos.x)/RENDER_DISTANCE*2.0;}}

//Shadow water

if(uv1.y<0.875){
diffuse.rgb *= vec3(0.8);
}

}
#endif

//Torch

if(FOG_CONTROL.x == 0.0 && FOG_COLOR.b > FOG_COLOR.r){
diffuse.rgb += vec3(0.0,0.5,0.2)*pow(uv1.x*1.2,1.0);
}else{
vec2 tjv = vec2(0.83) - uv1*0.86;
float jarak = 1./length(tjv.x);
jarak *= 0.2;
jarak = pow(jarak,1.2);
vec3 torch_color = jarak * vec3(1.0,0.4,0.0)*0.3;
torch_color = 1.-exp(-torch_color);
diffuse.rgb += torch_color * uv1.x;
}

#ifdef FOG

//Underwater

if(FOG_CONTROL.x == 0.0 && FOG_COLOR.b > FOG_COLOR.r){
diffuse.rgb += clamp(abs(wave)*vec3(0.01),0.0,1.6);
diffuse.rgb += noise(wpos*0.3*abs(wave))*0.22;
diffuse.rgb *= mix(vec3(0.7),vec3(0.7),abs(wave));
diffuse.rgb += diffuse.rgb*(vec3(0.0,0.0,1.5)*1.15)*pow(uv1.x*1.6,1.0);
}
#endif

//Fog nether

if(nether==true){

vec3 nether_fog = mix(diffuse.rgb, FOG_COLOR.rgb*vec3(5.5,5.5,5.5), clamp(length(-wpos)/RENDER_DISTANCE*1.0,0.0,0.4));

diffuse.rgb = nether_fog;
}

if(soul_nether==true){
	
vec3 nether_fog = mix(diffuse.rgb, FOG_COLOR.rgb*vec3(2.5,2.5,2.5), clamp(length(-wpos)/RENDER_DISTANCE*0.4,0.0,0.4));

diffuse.rgb = nether_fog;

}

if(unwater==true){
diffuse.rgb *= vec3(0.4,0.5,0.63);

}

//Sunbeam by t1ap#1613 (Creator RUSPE shader)

if(nether==false){
diffuse.rgb = 
mix(diffuse.rgb, mix(
diffuse.rgb,
FOG_COLOR.rgb-wpos.y*0.000005,
clamp(length(-wpos)/
RENDER_DISTANCE*5.8, 
0.0, 1.0)), FOG_COLOR.a*-wpos.y*0.014);

float s = 1.0;


float bm = abs(wpos.y);
	if(bm > -4. && bm < 4.){ bm = map(bm, -4., 4., 0., 1.); }else{bm = 1.; }
	
	float sunBeam = pow(map(sin(atan(wpos.y, wpos.z)/  (3.141532 * 2.) * 75.1), -1., 1., 0., 1.), 1.75) * 5.75;
	//float sunBeam = 1.;
sunBeam *= (((cotf.r+0.1)-cotf.b));
	if(sunBeam >= 1.5){ sunBeam=1.5; }
	sunBeam*=bm*s;
	
	if(sunBeam >= 1.){ diffuse.rgb *= sunBeam; }}

vec2 render = uv1;
render.y -= playershadow()*0.51;
diffuse *= texture2D(TEXTURE_1,render);

diffuse.rgb = ToneMap(diffuse.rgb);
diffuse.rgb = saturate(diffuse.rgb,1.5);

#ifdef FOG
if(unwater==true){
	diffuse.rgb = mix( diffuse.rgb, fogColor.rgb, fogColor.a );}
#endif

	gl_FragColor = diffuse;
	
#endif // BYPASS_PIXEL_SHADER
}